#!/usr/bin/env python

# Copyright 2016 NeuroData (http://neurodata.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# qa_fibers.py
# Created by Vikram Chandrashekhar.
# Email: Greg Kiar @ gkiar@jhu.edu

import numpy as np
import nibabel as nib
import random
import sys
import os
import re
import vtk
import paramiko
import getpass
import subprocess

from dipy.viz import window, actor
from argparse import ArgumentParser


def visualize(fibfile, atlasfile, outdir, opacity, num_samples, fname=None):
    """
    Takes fiber streamlines and visualizes them using DiPy
    Required Arguments:
        - fibfile: Path to fiber file
        - atlasfile: Path to atlas file
        - outdir: Path to output directory
        - opacity: Opacity of overlayed brain
        - num_samples: number of fibers to randomly sample from fibfile
    Optional Arguments:
        - fname: name of output file. default is None (fname based on input
          fibfile name)
    """
    # loading the fibers
    fibs = np.load(fibfile)
    fibs = fibs[fibs.keys()[0]]
    fibs = threshold_fibers(fibs)

    # make sure if fiber streamlines
    # have no fibers, no error occurs
    if len(fibs) == 0:
        return
    # randomly sample num_samples fibers from given fibers
    resampled_fibs = random_sample(fibs, num_samples)

    # load atlas file
    atlas_volume = load_atlas(atlasfile, opacity)

    # Initialize renderer
    renderer = window.Renderer()

    # Add streamlines as a DiPy viz object
    stream_actor = actor.line(fibs)

    # Set camera orientation properties
    # TODO: allow this as an argument
    renderer.set_camera()  # args are: position=(), focal_point=(), view_up=()

    # Add streamlines to viz session
    renderer.add(stream_actor)
    renderer.add(atlas_volume)

    # Display fibers
    # TODO: allow size of window as an argument
    # window.show(renderer, size=(600, 600), reset_camera=False)

    # Saves file, if you're into that sort of thing...
    if fname is None:
        fname = os.path.split(fibfile)[1].split('.')[0] + '.png'
    window.record(renderer, out_path=outdir + fname, size=(600, 600))
    print('done')


def threshold_fibers(fibs):
    '''
    fibs: fibers as 2D array (N,3)
    '''
    fib_lengths = [len(f) for f in fibs]
    if (len(fib_lengths) == 0):
        return fib_lengths
    # calculate median of  fiber lengths
    med = np.median(fib_lengths)
    # get only fibers above the median length
    long_fibs = [f for f in fibs if len(f) > med]
    return long_fibs


def random_sample(fibs, num_samples):
    '''
    fibs: fibers thresholded above median
    num_samples: number of fibers to sample from fibs
    '''
    # if the number of samples is more than amount
    # of fibers available, then make num_samples
    # equal number of fibers available
    if (len(fibs) <= num_samples):
        num_samples = len(fibs)
    # generate the random sample indices
    samples = random.sample(range(len(fibs)), num_samples)
    return [fibs[i] for i in samples]


def load_atlas(path, opacity):
    '''
    path: path to atlas file
    opacity: opacity of overlayed atlas brain
    '''
    nifti_reader = vtk.vtkNIFTIImageReader()
    nifti_reader.SetFileName(path)
    nifti_reader.Update()

    # The following class is used to store transparencyv-values for later
    # retrival. In our case, we want the value 0 to be completly opaque
    alphaChannelFunc = vtk.vtkPiecewiseFunction()
    alphaChannelFunc.AddPoint(0, 0.0)
    alphaChannelFunc.AddPoint(1, opacity)

    # This class stores color data and can create color tables from a few color
    # points. For this demo, we want the three cubes to be of the colors red
    # green and blue.
    colorFunc = vtk.vtkColorTransferFunction()
    colorFunc.AddRGBPoint(0, 0.0, 0.0, 0.0)
    colorFunc.AddRGBPoint(1, 1.0, 1.0, 1.0)

    # The preavius two classes stored properties. Because we want to apply
    # these properties to the volume we want to render, we have to store them
    # in a class that stores volume prpoperties.
    volumeProperty = vtk.vtkVolumeProperty()
    volumeProperty.SetColor(colorFunc)
    volumeProperty.SetScalarOpacity(alphaChannelFunc)
    volumeProperty.ShadeOn()

    # We can finally create our volume. We also have to specify the data for
    # it, as well as how the data will be rendered.
    volumeMapper = vtk.vtkSmartVolumeMapper()
    volumeMapper.SetInputDataObject(nifti_reader.GetOutput())

    # The class vtkVolume is used to pair the preaviusly declared volume as
    # well as the properties to be used when rendering that volume.
    volume = vtk.vtkVolume()
    volume.SetMapper(volumeMapper)
    volume.SetProperty(volumeProperty)

    return volume


def main():
    """
    Argument parser. Takes organization and atlas
    information and produces a dictionary of file lists based on datasets
    of interest and then passes it off for processing.
    Required parameters:
        fibfile:
            - Path to tensor file
        atlasfile:
            - Path to regdti file
        opacity:
            - Opacity of overlayed atlas file
        outdir:
            - Path to fa png save location
    """
    parser = ArgumentParser(description="Generates a fiber png based on fibers"
                            " and atlas")
    parser.add_argument("fibfile", action="store", help="base directory loc")
    parser.add_argument("atlasfile", action="store", help="base directory loc")
    parser.add_argument("outdir", action="store", help="base directory loc")
    parser.add_argument("--opacity", action="store", help="opacity of"
                        " overlayed atlas file", type=float, default=0.05)
    parser.add_argument("--samples", action="store", help="change number of"
                        " fibers to show in png", type=int, default=4000)
    result = parser.parse_args()

    visualize(result.fibfile, result.atlasfile, result.outdir, result.opacity,
              result.samples)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Copyright 2016 NeuroData (http://neurodata.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# ndmg_cloud.py
# Created by Greg Kiar on 2017-02-02.
# Email: gkiar@jhu.edu

from argparse import ArgumentParser
from copy import deepcopy
import ndmg.utils as mgu
import ndmg
import sys
import os
import re
import boto3
import json

job_template = 'https://raw.githubusercontent.com/neurodata/sic/master/code/ec2/batch/json_files/job_template.json'

def batch_submit(bucket, path, jobdir, debug=False, dataset=None):
    """
    Searches through an S3 bucket, gets all subject-ids, creates json files
    for each, submits batch jobs, and returns list of job ids to query status
    upon later.
    """
    print("Getting subject list from s3://{}/{}/...".format(bucket, path))
    subjs = crawl_bucket(bucket, path)
    print("Generating job for each subject...")
    jobs = create_json(bucket, path, subjs, jobdir, debug, dataset)
    print("Submitting jobs to the queue...")
    ids = submit_jobs(jobs) 

def crawl_bucket(bucket, path):
    """
    Gets subject list for a given S3 bucket and path
    """
    cmd = 'aws s3 ls s3://{}/{}/'.format(bucket, path)
    out, err = mgu().execute_cmd(cmd)
    subjs = re.findall('sub-(.+)/', out)
    print("Subject IDs: " + ", ".join(subjs)) 
    return subjs


def create_json(bucket, path, subjs, jobdir, debug=False, dataset=None):
    """
    Takes parameters to make jsons
    """
    mgu().execute_cmd("mkdir -p {}".format(jobdir))
    mgu().execute_cmd("mkdir -p {}/jobs/".format(jobdir))
    if not os.path.isfile('{}/job_template.json'.format(jobdir)):
        cmd = 'wget --quiet -P {} {}'.format(jobdir, job_template)
        mgu().execute_cmd(cmd)

    with open('{}/job_template.json'.format(jobdir), 'r') as infile:
        template = json.load(infile)
    cmd = template['containerOverrides']['command']

    print(subjs)
    jobs = list()
    for idx, subj in enumerate(subjs):
        print("... Generating job for sub-{}".format(subj))
        print(subjs)
        job_cmd = deepcopy(cmd)
        job_cmd[4] = re.sub('(<BUCKET>)', bucket, job_cmd[4])
        job_cmd[6] = re.sub('(<PATH>)', path, job_cmd[6])
        job_cmd[8] = re.sub('(<SUBJ>)', subj, job_cmd[8])
        if debug:
            job_cmd += [u'--debug']
        
        job_json = deepcopy(template)
        ver = ndmg.version.replace('.', '-')
        if dataset:
            name = 'ndmg-{}_{}_sub-{}'.format(ver, dataset, subj)
        else:
            name = 'ndmg_{}_sub-{}'.format(ver, subj)
        job_json['jobName'] = name
        job_json['containerOverrides']['command'] = job_cmd
        
        jobs += [os.path.join(jobdir, 'jobs', name)]
        with open(jobs[idx], 'w') as outfile:
            json.dump(job_json, outfile)
    return jobs


def submit_jobs(jobs):
    """
    Give list of jobs to submit, submits them to AWS Batch
    """
    cmd_template = 'aws batch submit-job --cli-input-json file://{}'

    for job in jobs:
        cmd = cmd_template.format(job)
        print("... Submitting job {}...".format(job))
        mgu().execute_cmd(cmd)

    return 0


def get_status(jobdir):
    """
    Given list of jobs, returns status of each.
    """
    for job in jobs:
        pass
        # TODO: this
    return 0


def main():
    parser = ArgumentParser(description="This is an end-to-end connectome \
                            estimation pipeline from sMRI and DTI images")

    parser.add_argument('--bucket', help='The S3 bucket with the input dataset'
                        ' formatted according to the BIDS standard.')
    parser.add_argument('--bidsdir', help='The directory where the dataset'
                        ' lives on the S3 bucket should be stored. If you'
                        ' level analysis this folder should be prepopulated'
                        ' with the results of the participant level analysis.')
    parser.add_argument('--jobdir', action='store', help='Dir of batch jobs to'
                        ' generate/check up on.')
    parser.add_argument('--debug', action='store_true', help='flag to store '
                        'temp files along the path of processing.',
                        default=False)
    parser.add_argument('--status', action='store_true', help='flag to check'
                        'status of jobs rather than launch them.',
                        default=False)
    parser.add_argument('--dataset', action='store', help='Dataset name')
    result = parser.parse_args()

    bucket = result.bucket
    remo = result.bidsdir
    debug = result.debug
    status = result.status
    jobdir = result.jobdir
    dset = result.dataset
    if jobdir is None:
        jobdir = './'

    if (bucket is None or remo is None) and (status is False):
        sys.exit('Requires either path to bucket and data, or the status flag'
                 ' and job IDs to query. Try ndmg_cloud --help')
    
    if status:
        print("Checking job status...")
        get_status(jobdir)
    else:
        print("Beginning batch submission process...")
        batch_submit(bucket, remo, jobdir, debug, dset)
    
    sys.exit(0) 

if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Copyright 2016 NeuroData (http://neurodata.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# qa_regdti.py
# Created by Vikram Chandrashekhar.
# Email: Greg Kiar @ gkiar@jhu.edu

import os
import re
import sys
import numpy as np
import nibabel as nib
from argparse import ArgumentParser
from matplotlib.colors import colorConverter
from skimage.filters import threshold_otsu
from scipy import ndimage
from matplotlib.colors import LinearSegmentedColormap
from dipy.core.gradients import gradient_table
from dipy.io import read_bvals_bvecs
import matplotlib as mpl
mpl.use('Agg')  # very important above pyplot import

import matplotlib.pyplot as plt


def save_reg_pngs(fs, outdir, fname=None):
    """
    fs: 4-tuple of paths to regdti, bval, bvec, and atlas files
    outdir: directory where output png file is saved
    fname: name of output file WITHOUT FULL PATH. Path provided in outdir.
    """
    plt.rcParams.update({'axes.labelsize': 'x-large',
                         'axes.titlesize': 'x-large'})
    # unpack the files from the tuple
    fdti, fbval, fbvec, fatlas = fs
    # load atlas file and extract data
    atlas_img = nib.load(fatlas)
    atlas_data = atlas_img.get_data()
    # set the slice numbers you want to visualize
    # for each of the 3 dimensions
    x1 = 78
    x2 = 90
    x3 = 100
    y1 = 82
    y2 = 107
    y3 = 142
    z1 = 88
    z2 = 103
    z3 = 107
    # load dti file and extract data
    img = nib.load(fdti)
    data = img.get_data()
    # making sure the image displayed is the
    # brain without any magnetic gradent applied
    data = get_b0_vol(data, fbval, fbvec)

    # creating the two custom colormaps
    cmap1 = LinearSegmentedColormap.from_list('mycmap1', ['black', 'magenta'])
    cmap2 = LinearSegmentedColormap.from_list('mycmap2', ['black', 'green'])

    # create subplot for first slice
    # and customize all labels
    ax_x1 = plt.subplot(331)
    ax_x1.set_ylabel('Sagittal Slice: Y and Z fixed')
    ax_x1.set_title('X = ' + str(x1))
    ax_x1.yaxis.set_ticks([0, data.shape[2]/2, data.shape[2] - 1])
    ax_x1.xaxis.set_ticks([0, data.shape[1]/2, data.shape[1] - 1])
    image = data[x1, :, :, 0]
    # get the intensity value of the 3rd  and 98th percentiles (as min_val
    # and max_val, respectively) important to reduce impact of outliers on
    # intensity of image
    min_val, max_val = get_min_max(image)
    # displaying the images with atlas image using magenta and regdti using
    # green colormaps
    plt.imshow(ndimage.rotate(atlas_data[x1, :, :], 90), interpolation='none',
               cmap=cmap1, alpha=0.5)
    plt.imshow(ndimage.rotate(image, 90), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)
    # repeating above for each of the following slices
    ax_x2 = plt.subplot(332)
    ax_x2.set_title('X = ' + str(x2))
    ax_x2.get_yaxis().set_ticklabels([])
    ax_x2.get_xaxis().set_ticklabels([])
    image = data[x2, :, :, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(ndimage.rotate(atlas_data[x2, :, :], 90), interpolation='none',
               cmap=cmap1, alpha=0.5)
    plt.imshow(ndimage.rotate(image, 90), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)
    ax_x3 = plt.subplot(333)
    ax_x3.set_title('X = ' + str(x3))
    ax_x3.get_yaxis().set_ticklabels([])
    ax_x3.get_xaxis().set_ticklabels([])
    image = data[x3, :, :, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(ndimage.rotate(atlas_data[x3, :, :], 90), interpolation='none',
               cmap=cmap1, alpha=0.5)
    plt.imshow(ndimage.rotate(image, 90), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)
    ax_y1 = plt.subplot(334)
    ax_y1.set_ylabel('Coronal Slice: X and Z fixed')
    ax_y1.set_title('Y = ' + str(y1))
    ax_y1.yaxis.set_ticks([0, data.shape[0]/2, data.shape[0] - 1])
    ax_y1.xaxis.set_ticks([0, data.shape[2]/2, data.shape[2] - 1])
    image = data[:, y1, :, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(ndimage.rotate(atlas_data[:, y1, :], 90), interpolation='none',
               cmap=cmap1, alpha=0.5)
    plt.imshow(ndimage.rotate(image, 90), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)
    ax_y2 = plt.subplot(335)
    ax_y2.set_title('Y = ' + str(y2))
    ax_y2.get_yaxis().set_ticklabels([])
    ax_y2.get_xaxis().set_ticklabels([])
    image = data[:, y2, :, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(ndimage.rotate(atlas_data[:, y2, :], 90), interpolation='none',
               cmap=cmap1, alpha=0.5)
    plt.imshow(ndimage.rotate(image, 90), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)
    ax_y3 = plt.subplot(336)
    ax_y3.set_title('Y = ' + str(y3))
    ax_y3.get_yaxis().set_ticklabels([])
    ax_y3.get_xaxis().set_ticklabels([])
    image = data[:, y3, :, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(ndimage.rotate(atlas_data[:, y3, :], 90), interpolation='none',
               cmap=cmap1, alpha=0.5)
    plt.imshow(ndimage.rotate(image, 90), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)
    ax_z1 = plt.subplot(337)
    ax_z1.set_ylabel('Axial Slice: X and Y fixed')
    ax_z1.set_title('Z = ' + str(z1))
    ax_z1.yaxis.set_ticks([0, data.shape[0]/2, data.shape[0] - 1])
    ax_z1.xaxis.set_ticks([0, data.shape[1]/2, data.shape[1] - 1])
    image = data[:, :, z1, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(atlas_data[:, :, z1], interpolation='none', cmap=cmap1,
               alpha=0.5)
    plt.imshow(ndimage.rotate(image, 0), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)

    ax_z2 = plt.subplot(338)
    ax_z2.set_title('Z = ' + str(z2))
    ax_z2.get_yaxis().set_ticklabels([])
    ax_z2.get_xaxis().set_ticklabels([])
    image = data[:, :, z2, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(atlas_data[:, :, z2], interpolation='none', cmap=cmap1,
               alpha=0.5)
    plt.imshow(ndimage.rotate(image, 0), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)

    ax_z3 = plt.subplot(339)
    ax_z3.set_title('Z = ' + str(z3))
    ax_z3.get_yaxis().set_ticklabels([])
    ax_z3.get_xaxis().set_ticklabels([])
    image = data[:, :, z3, 0]
    min_val, max_val = get_min_max(image)
    plt.imshow(atlas_data[:, :, z3], interpolation='none', cmap=cmap1,
               alpha=0.5)
    plt.imshow(ndimage.rotate(image, 0), interpolation='none', cmap=cmap2,
               vmin=min_val, vmax=max_val, alpha=0.5)
    # getting the current figure which has all the slices plotted on it
    fig = plt.gcf()
    fig.set_size_inches(12.5, 10.5, forward=True)
    # name and save the file
    if fname is None:
        fname = os.path.split(fdti)[1].split(".")[0] + '.png'
    plt.savefig(outdir + '/' + fname, format='png')
    print(fname + " saved!")


def get_min_max(data):
    '''
    data: regdti data to threshold.
    '''
    min_val = np.percentile(data, 3)
    max_val = np.percentile(data, 92)
    return (min_val, max_val)


def get_b0_vol(data, fbval, fbvec):
    '''
    data: 4D regdti data
    fbval: path to bval file
    fbvec: path to bvec file
    '''
    bvals, bvecs = read_bvals_bvecs(fbval, fbvec)
    # make sure that all bvecs are unit vectors
    normalize_bvecs(bvecs)
    gtab = gradient_table(bvals, bvecs)
    # return the 0-gradient volume
    return data[:, :, :, gtab.b0s_mask]


def normalize_bvecs(bvecs):
    '''
    bvecs: (N,3) array of bvecs
    '''
    for i in range(bvecs.shape[0]):
        norm = np.linalg.norm(bvecs[i, :])
        # if not unit length, normalize
        if norm is not 1 or norm is not 0:
            bvecs[i, :] = bvecs[i, :]/norm


def main():
    """
    Argument parser.
    Required parameters:
        dtifile:
            - path to regdti file
        bvalfile:
            - path to bval file
        bvecfile:
            - path to bvec file
        atlasfile:
            - path to atlas file
        outdir:
            - output file save directory
    """
    parser = ArgumentParser(description="Generates registration QA images")
    parser.add_argument("dtifile", action="store", help="base directory loc")
    parser.add_argument("bvalfile", action="store", help="base directory loc")
    parser.add_argument("bvecfile", action="store", help="base directory loc")
    parser.add_argument("atlasfile", action="store", help="base directory loc")
    parser.add_argument("outdir", action="store", help="base directory loc")
    result = parser.parse_args()

    # create the output directory if it doesn't exist
    if (not os.path.isdir(result.outdir)):
        os.mkdir(result.outdir)
    # do all the processing and save the output png file
    save_reg_pngs((result.dtifile, result.bvalfile, result.bvecfile,
                   result.atlasfile), result.outdir)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2016 NeuroData (http://neurodata.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# qa_tensor.py
# Created by Vikram Chandrashekhar.
# Email: Greg Kiar @ gkiar@jhu.edu

from dipy.reconst.dti import fractional_anisotropy, color_fa
from argparse import ArgumentParser
from scipy import ndimage
import os
import re
import numpy as np
import nibabel as nib
import sys
import matplotlib

matplotlib.use('Agg')  # very important above pyplot import
import matplotlib.pyplot as plt


def tensor2fa(fs, outdir, fname=None):
    '''
    fs: of 2-tuples containing the tensor file and regdti file
    outdir: location of output directory.
    fname: name of output fa map file. default is none (name created based on
    input file)
    '''
    # load dti file
    ften, fdti = fs
    img = nib.load(fdti)
    dti_data = img.get_data()

    # load tensor file and grab data
    with np.load(ften) as data:
        tensor_fit = data['arr_0']
        tensor_fit = tensor_fit.tolist()

    # create FA map
    FA = fractional_anisotropy(tensor_fit.evals)
    # get rid of NaNs
    FA[np.isnan(FA)] = 0
    # check if fname is defined
    if fname is None:
        fname = os.path.split(ften)[1].split(".")[0] + '_fa_rgb.nii.gz'
    if outdir is None:
        outdir = './'
    # generate the RGB FA map
    FA = np.clip(FA, 0, 1)
    RGB = color_fa(FA, tensor_fit.evecs)
    # save the RGB FA map
    nib.save(nib.Nifti1Image(np.array(255 * RGB, 'uint8'), img.affine),
             outdir + fname)
    # return path of file
    return outdir + fname


def save_fa_png(fa_file, outdir, fname=None):
    '''
    fa_file: path to fa map file
    '''
    plt.rcParams.update({'axes.labelsize': 'x-large',
                         'axes.titlesize': 'x-large'})

    # helper function to make sure that slices are plotted on the same figure
    def create_subplot(number, title, ylabel=None, xlabel=None, set_ticks=None,
                       set_ticklabels=False):
        ax = plt.subplot(number)
        ax.set_title(title)
        if ylabel is not None:
            ax.set_ylabel(ylabel)
        if xlabel is not None:
            ax.set_xlabel(xlabel)
        if set_ticks is not None:
            ax.xaxis.set_ticks(set_ticks[0])
            ax.yaxis.set_ticks(set_ticks[1])
        if set_ticklabels:
            ax.get_yaxis().set_ticklabels([])
            ax.get_xaxis().set_ticklabels([])
        return ax
    img = nib.load(fa_file)
    data = img.get_data()
    ax_x1 = create_subplot(331, 'X = 78',
                           ylabel='Sagittal Slice: Y and Z fixed',
                           set_ticks=([0, data.shape[1]/2, data.shape[1] - 1],
                                      [0, data.shape[2]/2, data.shape[2] - 1]))
    plt.imshow(ndimage.rotate(data[78, :, :], 90))
    ax_x2 = create_subplot(332, 'X = 90', set_ticklabels=True)
    plt.imshow(ndimage.rotate(data[90, :, :], 90))
    ax_x3 = create_subplot(333, 'X = 100', set_ticklabels=True)
    plt.imshow(ndimage.rotate(data[100, :, :], 90))
    ax_y1 = create_subplot(334, 'Y = 82',
                           ylabel='Coronal Slice: X and Z fixed',
                           set_ticks=([0, data.shape[0]/2, data.shape[0] - 1],
                                      [0, data.shape[2]/2, data.shape[2] - 1]))
    plt.imshow(ndimage.rotate(data[:, 82, :], 90))
    ax_y2 = create_subplot(335, 'Y = 107', set_ticklabels=True)
    plt.imshow(ndimage.rotate(data[:, 107, :], 90))
    ax_y3 = create_subplot(336, 'Y = 142', set_ticklabels=True)
    plt.imshow(ndimage.rotate(data[:, 142, :], 90))
    ax_z1 = create_subplot(337, 'Z = 88',
                           ylabel='Axial Slice: X and Y fixed',
                           set_ticks=([0, data.shape[0]/2, data.shape[0] - 1],
                                      [0, data.shape[1]/2, data.shape[1] - 1]))
    plt.imshow(data[:, :, 88])
    ax_z2 = create_subplot(338, 'Z = 103', set_ticklabels=True)
    plt.imshow(data[:, :, 103])
    ax_z3 = create_subplot(339, 'Z = 107', set_ticklabels=True)
    plt.imshow(data[:, :, 107])
    fig = plt.gcf()
    fig.set_size_inches(12.5, 10.5, forward=True)
    if fname is None:
        fname = os.path.split(fa_file)[1].split(".")[0] + '.png'
    plt.savefig(outdir + fname, format='png')
    print(fname + " saved!")


def main():
    """
    Argument parser.
    Required parameters:
        tenfile:
            - Path to tensor file
        dtifile:
            - Path to regdti file
        faoutdir:
            - Path to fa map save location
        pngoutdir:
            - Path to fa png save location
    """
    parser = ArgumentParser(description="Computes FA maps and saves FA slices")
    parser.add_argument("tenfile", action="store", help="path to tensor file")
    parser.add_argument("dtifile", action="store", help="path to regdti file")
    parser.add_argument("faoutdir", action="store", help="output directory"
                        " for fa map")
    parser.add_argument("pngoutdir", action="store", help="output directory"
                        " for png")
    result = parser.parse_args()

    if (not os.path.isdir(result.faoutdir)):
        os.mkdir(result.faoutdir)
    if (not os.path.isdir(result.pngoutdir)):
        os.mkdir(result.pngoutdir)
    fafile = tensor2fa((result.tenfile, result.dtifile), result.faoutdir)
    save_fa_png(fafile, result.pngoutdir)

if __name__ == '__main__':
    main()
